<%@ WebHandler Language="C#" Class="WebHookHandlerExample" %>

using System;
using System.Web;
using System.IO;
using System.Text;
using REPSolution.OFSYS.Objects.Webhooks;
using REPSolution.OFSYS.Objects.Security;
using System.Security.Cryptography;

public class WebHookHandlerExample : IHttpHandler
{
	// Paste the Dialog Insight public key (XML format) here
	const string xmlPublicKey = @"<RSAKeyValue>
<Modulus>vhvYWPrp0sneb0sz3QpafHeoXOU8b1chV+LM+405S7mT3B0GwIrtpF9MbJmKjhfP3AzqUfUeNC/WYIDqZZvcwwcBfEHP0929oKwf0kpdJf3jpeh3D5RrKBjq0E8l0PWQK5p13W4YDKHEjg6MHHOsVrKK511qVF5SUQj6usGiPIM=</Modulus>
<Exponent>AQAB</Exponent>
</RSAKeyValue>";

	public void ProcessRequest (HttpContext context)
	{
		// Read the posted JSON body
		string json;
		using (StreamReader stream = new System.IO.StreamReader(context.Request.InputStream, Encoding.UTF8))
		{
			json = stream.ReadToEnd();
		}

		// Get the "X-DI-Signature" header value
		string signature = context.Request.Headers["X-DI-Signature"];
		// Validate the signature with json data
		bool isSignatureValid = VerifySignature(json, signature);

		if (isSignatureValid)
		{
			// Do not process event contents here : queue the event to be processed asychronously.
			// Exemple: dumping to a file:
			string filename = String.Format(@"C:\Temp\webhook-event-{0}.json", Guid.NewGuid().ToString());
			File.WriteAllText(filename, json);
		}
		else
		{
			// TODO: Error handling, invalid signature
		}
	}

	// Function to validate
	protected bool VerifySignature(string json, string signatureHeader)
	{
		if (String.IsNullOrWhiteSpace(signatureHeader))
			return false;

		RSACryptoServiceProvider rsaProvider = new RSACryptoServiceProvider();
		rsaProvider.FromXmlString(xmlPublicKey);

		byte[] bJsonPayload = Encoding.UTF8.GetBytes(json);
		byte[] bSignature = Convert.FromBase64String(signatureHeader);
		bool isSignatureValid;

		try
		{
			isSignatureValid = rsaProvider.VerifyData(bJsonPayload, new SHA256CryptoServiceProvider(), bSignature);
		}
		catch (CryptographicException)
		{
			isSignatureValid = false;
		}

		return isSignatureValid;
	}

	public bool IsReusable
	{
		get { return false; }
	}
}
